Module process_result

Module process_result 

Structs

ProcessResults
A future that processes results from an asynchronous iterator, collecting successes and errors based on the specified strategy.
ProcessResultsContainer
A container that holds both successful and erroneous results.

Enums

ProcessResultsStrategy
Defines the strategy to use when processing results from an asynchronous iterator.