Structs
- Process
Results - A future that processes results from an asynchronous iterator, collecting successes and errors based on the specified strategy.
- Process
Results Container - A container that holds both successful and erroneous results.
Enums
- Process
Results Strategy - Defines the strategy to use when processing results from an asynchronous iterator.