pub struct ProcessResults<I, T, E>where
I: AsyncIterator<Item = Result<T, E>>,{ /* private fields */ }Expand description
A future that processes results from an asynchronous iterator, collecting successes and errors based on the specified strategy.
Implementations
Sourceimpl<I, T, E> ProcessResults<I, T, E>where
I: AsyncIterator<Item = Result<T, E>>,
impl<I, T, E> ProcessResults<I, T, E>where
I: AsyncIterator<Item = Result<T, E>>,
Sourcepub fn new(iter: I) -> ProcessResults<I, T, E> ⓘ
pub fn new(iter: I) -> ProcessResults<I, T, E> ⓘ
Constructs a new ProcessResults future using the given async iterator.
Sourcepub fn with_process_strategy(self, strategy: ProcessResultsStrategy) -> Self
pub fn with_process_strategy(self, strategy: ProcessResultsStrategy) -> Self
Sets the processing strategy to use for handling errors during iteration.